<?php

namespace Drupal\book_library_api\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Database\Connection;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;

/**
 * Controller for Book Author Autocomplete.
 */
class BookController extends ControllerBase
{
  /**
   * The database connection.
   * 
   * @var \Drupal\Core\Database\Connection
   */
  protected $database;

  /**
   * Constructs a BookController object.
   *
   * @param \Drupal\Core\Database\Connection $database
   *   The database connection.
   */
  public function __construct(Connection $database)
  {
    $this->database = $database;
  }

  /**
   * {@inheritdoc}
   */
  public static function createInstance(ContainerInterface $container)
  {
    return new self(
      $container->get('database')
    );
  }

  /**
   * Callback for book author autocomplete.
   */
  public function bookAuthorAutocomplete(Request $request)
  {

    $results = [];
    $string = $request->query->get('q');

    $query = $this->database->select('book', 'b');
    $query->fields('b', ['field_book_author']);
    $query->distinct();
    $query->condition('field_book_author', '%' . $string . '%', 'LIKE');
    $query->range(0, 10);
    $authors = $query->execute()->fetchAll(\PDO::FETCH_COLUMN);

    foreach ($authors as $author) {
      $results[] = [
        'value' => $author,
        // 'label' => $author,
      ];
    }

    return new JsonResponse($results);
  }

  /**
   * Callback for book title autocomplete.
   */
  public function bookTitleAutocomplete(Request $request)
  {
    $results = [];
    $string = $request->query->get('q');

    $query = $this->database->select('book', 'b');
    $query->fields('b', ['field_book_title']);
    $query->distinct();
    $query->condition('field_book_title', '%' . $string . '%', 'LIKE');
    $query->range(0, 10);
    $titles = $query->execute()->fetchAll(\PDO::FETCH_COLUMN);

    foreach ($titles as $title) {
      $results[] = [
        'value' => $title,
        // 'label' => $title,
      ];
    }

    return new JsonResponse($results);
  }

}