<?php

declare(strict_types=1);

namespace Drupal\book_library_api\Form;

use Drupal\Core\Entity\ContentEntityForm;
use Drupal\Core\Form\FormStateInterface;


/**
 * Form controller for the book entity edit forms.
 */
final class BookForm extends ContentEntityForm
{
  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state)
  {
    parent::validateForm($form, $form_state);

    // Validate the title field to ensure it is unique.
    $title = $form_state->getValue('field_book_title')[0]['value'];

    /** @var \Drupal\Core\Entity\ContentEntityInterface $entity */
    $entity = $this->getEntity();

    $query = $this->entityTypeManager->getStorage('book')->getQuery();
    $query->condition('field_book_title', $title);

    // Exclude the current entity from the query if it's an edit form.
    if (!$entity->isNew()) {
      $query->condition('id', $entity->id(), '<>');
    }

    // Explicitly set access check.
    $query->accessCheck(FALSE);

    $ids = $query->execute();

    if (!empty($ids)) {
      $form_state->setErrorByName('field_book_title', $this->t('The book title %title already exists.', ['%title' => $title]));
    }

    return $entity;
  }


  /**
   * {@inheritdoc}
   */
  public function save(array $form, FormStateInterface $form_state): int
  {
    $result = parent::save($form, $form_state);

    $message_args = ['%label' => $this->entity->toLink()->toString()];
    $logger_args = [
      '%label' => $this->entity->get('field_book_title')->value,
      'link' => $this->entity->toLink($this->t('View'))->toString(),
    ];

    switch ($result) {
      case SAVED_NEW:
        $this->messenger()->addStatus($this->t('New book %label has been created.', $message_args));
        $this->logger('book_library_api')->notice('New book %label has been created.', $logger_args);
        break;

      case SAVED_UPDATED:
        $this->messenger()->addStatus($this->t('The book %label has been updated.', $message_args));
        $this->logger('book_library_api')->notice('The book %label has been updated.', $logger_args);
        break;

      default:
        throw new \LogicException('Could not save the entity.');
    }

    $form_state->setRedirectUrl($this->entity->toUrl());

    return $result;
  }

}
